//+------------------------------------------------------------------+
//|                   Infinity Hull Moving Average (Multi) V1.00.mq5 |
//|                                  Copyright 2021, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2021, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"

#include <MovingAverages.mqh>

#property indicator_chart_window
#property indicator_plots   2
#property indicator_buffers 10


//--- Plot Label1
#property indicator_label1  "Hull Moving Average Color 1"
#property indicator_type1   DRAW_LINE
#property indicator_color1  clrRoyalBlue,clrGold
#property indicator_style1  STYLE_SOLID
#property indicator_width1  2
//--- Plot Label2
#property indicator_label2  "Hull Moving Average Color 2"
#property indicator_type2   DRAW_LINE
#property indicator_color2  clrMediumOrchid,clrWhite
#property indicator_style2  STYLE_SOLID
#property indicator_width2  2

//--- input parameters
input int                  HullPeriod1 = 224;
input ENUM_MA_METHOD       HullMethod1 = MODE_SMMA;
input ENUM_APPLIED_PRICE   HullPrice1  = PRICE_CLOSE;
input int                  HullShift1  = 0;
input int                  HullPeriod2 = 112;
input ENUM_MA_METHOD       HullMethod2 = MODE_SMMA;
input ENUM_APPLIED_PRICE   HullPrice2  = PRICE_CLOSE;
input int                  HullShift2  = 0;

//--- indicator buffers
double   HMA1[];
double   HMA1C[];
double   PeriodHalf1[];
double   PeriodFull1[];
double   HMACALCULATE1[];
//---    Buffer Second  Line
double   HMA2[];
double   HMA2C[];
double   PeriodHalf2[];
double   PeriodFull2[];
double   HMACALCULATE2[];
//--- Handels
int      HMA1_handle1;
int      HMA1_handle2;
int      HMA1_handle3;
//---
int      HMA2_handle1;
int      HMA2_handle2;
int      HMA2_handle3;


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,HMA1,INDICATOR_DATA);
   SetIndexBuffer(1,HMA2,INDICATOR_DATA);
   SetIndexBuffer(2,HMA1C,INDICATOR_COLOR_INDEX);
   SetIndexBuffer(3,HMA2C,INDICATOR_COLOR_INDEX);
   SetIndexBuffer(4,PeriodHalf1,INDICATOR_CALCULATIONS);
   SetIndexBuffer(5,PeriodFull1,INDICATOR_CALCULATIONS);
   SetIndexBuffer(6,HMACALCULATE1,INDICATOR_CALCULATIONS);
   SetIndexBuffer(7,PeriodHalf2,INDICATOR_CALCULATIONS);
   SetIndexBuffer(8,PeriodFull2,INDICATOR_CALCULATIONS);
   SetIndexBuffer(9,HMACALCULATE2,INDICATOR_CALCULATIONS);
   
   PlotIndexSetInteger(0,PLOT_COLOR_INDEXES,2);
   PlotIndexSetInteger(1,PLOT_COLOR_INDEXES,2);
   
   string linename1;
   StringConcatenate(linename1,"HMA(",HullPeriod1,")","(",EnumToString( HullMethod1 ),")");
   IndicatorSetString(INDICATOR_SHORTNAME,linename1);
    
   int halfperiod1 = HullPeriod1/2;
   int halfperiod2 = HullPeriod2/2;
   
   HMA1_handle1=iMA(Symbol(),0,HullPeriod1,HullShift1,HullMethod1,HullPrice1);
   HMA1_handle2=iMA(Symbol(),0,halfperiod1,HullShift1,HullMethod1,HullPrice1);
   HMA2_handle1=iMA(Symbol(),0,HullPeriod2,HullShift2,HullMethod2,HullPrice2);
   HMA2_handle2=iMA(Symbol(),0,halfperiod2,HullShift2,HullMethod2,HullPrice2);
   
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
//--- if the size of price[] is too small
   if(rates_total<HullPeriod1) return(0); // do not calculate or draw anything
//---   
   int sqrtperiod1 = (int)MathFloor(MathSqrt(HullPeriod1));
   int sqrtperiod2 = (int)MathFloor(MathSqrt(HullPeriod2));
   
//--- we can copy not all data
   int to_copy;
   if(prev_calculated>rates_total || prev_calculated<=0) to_copy=rates_total;
   else
     {
      to_copy=rates_total-prev_calculated;
      //--- last value is always copied
      to_copy++;
     }
//--- try to copy
   
   if(IsStopped()) // checking for stop flag
      return(0);
   if(CopyBuffer(HMA1_handle1,0,0,to_copy,PeriodFull1)<=0)
     {
      Print("Getting hma1handle1 is failed! Error ",GetLastError());
      return(0);
     }

//---
   if(IsStopped()) // checking for stop flag
      return(0);
   if(CopyBuffer(HMA1_handle2,0,0,to_copy,PeriodHalf1)<=0)
     {
      Print("Getting hma1handle2 is failed! Error ",GetLastError());
      return(0);
     }
   
//---
   if(IsStopped()) // checking for stop flag
      return(0);
   if(CopyBuffer(HMA2_handle1,0,0,to_copy,PeriodFull2)<=0)
     {
      Print("Getting hma2handle1 is failed! Error ",GetLastError());
      return(0);
     }

//---
   if(IsStopped()) // checking for stop flag
      return(0);
   if(CopyBuffer(HMA2_handle2,0,0,to_copy,PeriodHalf2)<=0)
     {
      Print("Getting hma2handle2 is failed! Error ",GetLastError());
      return(0);
     }
     
//--- now calculate the indicator values
   int start;
   
   if(prev_calculated==0) start=HullPeriod1-1; // set the starting index for input arrays
   else start=prev_calculated-1;    // set 'start' equal to the last index in the arrays
   for(int i=start;i<rates_total;i++)
     {
      HMACALCULATE1[i]=(2*PeriodHalf1[i])-(PeriodFull1[i]);
     }
//---
if(prev_calculated==0) start=HullPeriod2-1; // set the starting index for input arrays
   else start=prev_calculated-1;    // set 'start' equal to the last index in the arrays
   for(int i=start;i<rates_total;i++)
     {
      HMACALCULATE2[i]=(2*PeriodHalf2[i])-(PeriodFull2[i]);
     }     
//---
   if(HullMethod1 == MODE_SMA)
     {
      HMA1_handle3 = SimpleMAOnBuffer(rates_total,prev_calculated,0,sqrtperiod1,HMACALCULATE1,HMA1);
     }
   if(HullMethod1 == MODE_EMA)
     {
      HMA1_handle3 = ExponentialMAOnBuffer(rates_total,prev_calculated,0,sqrtperiod1,HMACALCULATE1,HMA1);
     }   
   if(HullMethod1 == MODE_SMMA)
     {
      HMA1_handle3 = SmoothedMAOnBuffer(rates_total,prev_calculated,0,sqrtperiod1,HMACALCULATE1,HMA1);
     }
   if(HullMethod1 == MODE_LWMA)
     {
      HMA1_handle3 = LinearWeightedMAOnBuffer(rates_total,prev_calculated,0,sqrtperiod1,HMACALCULATE1,HMA1);
     }                       
//---
   if(HullMethod2 == MODE_SMA)
     {
      HMA2_handle3 = SimpleMAOnBuffer(rates_total,prev_calculated,0,sqrtperiod2,HMACALCULATE2,HMA2);
     }
   if(HullMethod2 == MODE_EMA)
     {
      HMA2_handle3 = ExponentialMAOnBuffer(rates_total,prev_calculated,0,sqrtperiod2,HMACALCULATE2,HMA2);
     }   
   if(HullMethod2 == MODE_SMMA)
     {
      HMA2_handle3 = SmoothedMAOnBuffer(rates_total,prev_calculated,0,sqrtperiod2,HMACALCULATE2,HMA2);
     }
   if(HullMethod2 == MODE_LWMA)
     {
      HMA2_handle3 = LinearWeightedMAOnBuffer(rates_total,prev_calculated,0,sqrtperiod2,HMACALCULATE2,HMA2);
     }
//--- return value of prev_calculated for next call

   //--- main loop of calculations
   for(int a=0; a<rates_total-1 && !IsStopped(); a++)
     {
      HMA1C[a]=HMA1C[a+1];
      if (HMACALCULATE1[a] > HMACALCULATE1[a+1]) HMA1C[a] = 0;
      if (HMACALCULATE1[a] < HMACALCULATE1[a+1]) HMA1C[a] = 1;
      
      HMA2C[a]=HMA2C[a+1];
      if (HMACALCULATE2[a] > HMACALCULATE2[a+1]) HMA2C[a] = 1;
      if (HMACALCULATE2[a] < HMACALCULATE2[a+1]) HMA2C[a] = 0;
     }

   return(rates_total);
  }
//+------------------------------------------------------------------+
